import math
from ...utils.math import haveSameSign

def regulafalsi(fn, epsilon=0.1, a=-10, b=10):
  prevC = math.inf # infinity

  while True:
    fna = fn(a)
    fnb = fn(b)

    c = ((fna * b) - (fnb * a)) / (fna - fnb)
    fnc = fn(c)

    if (fnc == 0):
      return c

    if haveSameSign(fnc, fna):
      a = c
    elif abs(prevC - c) > epsilon:
      b = c
    else:
      return c

    prevC = c


from ...utils.math import haveSameSign

def graph(fn, dx=1, epsilon=0.1, x=0):
  prevX = x
  prevY = fn(x)

  while True:
    x += dx
    y = fn(x)

    if (y == 0):
      return x

    if haveSameSign(y, prevY):
      prevY = y
      prevX = x
    elif abs(x - prevX) > epsilon:
      x = prevX
      y = prevY
      dx /= 2
    else:
      return (x + prevX) / 2


from .utils import function
from .utils.function import fnx
from .nonlinear.bracketing.graph import graph
from .nonlinear.bracketing.bisection import bisection
from .nonlinear.bracketing.regulafalsi import regulafalsi


from functools import partial

def _fnx(degree, coefficients, baseExp, x):
  result = 0

  for exp in range(0, degree + 1):
    result += coefficients[degree - exp] * (x ** exp)

  return result ** baseExp

def fnx(degree, coefficients, baseExp):
  return partial(_fnx, degree, coefficients, baseExp)


from setuptools import setup, find_packages

long_description = ''

with open('README.md', 'r') as fh:
  long_description = fh.read()

setup(
  name='numerica',
  version='0.1.5',
  description='Numerical Analysis methods with Python (experimental)',
  long_description=long_description,
  long_description_content_type='text/markdown',
  url='http://github.com/ramesaliyev/numerica',
  author='Rames Aliyev',
  author_email='creator@ramesaliyev.com',
  license='MIT',
  packages=find_packages(),
  zip_safe=False)


import math
from ...utils.math import haveSameSign

def bisection(fn, epsilon=0.1, a=-10, b=10):
  prevC = math.inf # infinity

  while True:
    c = (a + b) / 2
    fnc = fn(c)

    if (fnc == 0):
      return c

    fna = fn(a)

    if haveSameSign(fnc, fna):
      a = c
    elif abs(prevC - c) > epsilon:
      b = c
    else:
      return c

    prevC = c